@@ -1,6 +1,7 @@
 use std::ops::Div;
 
 use anyhow::{Ok, anyhow};
+use itertools::Itertools;
 use polars::frame::DataFrame;
 use rand::Rng;
 
@@ -9,15 +10,17 @@ use crate::{
     app::{App, Content},
     misc::{
         globals::sql,
-        polars_ext::{InferDatetimeColumns, SafeInferSchema},
+        polars_ext::{IntoString, PlotData},
+        type_inferer::TypeInferer,
     },
     reader::{
         ArrowIpcToDataFrame, CsvToDataFrame, FwfToDataFrame, JsonLineToDataFrame, JsonToDataFrame,
         ParquetToDataFrame, ReadToDataFrames, Source, SqliteToDataFrames,
     },
     tui::{
-        TableType, TabularState, data_frame_table::DataFrameTableState, search_bar::SearchBarState,
-        tabular::Modal,
+        TabContentState, TableType, data_frame_info::DataFrameInfoState,
+        data_frame_table::DataFrameTableState, histogram_plot::HistogramPlotState,
+        scatter_plot::ScatterPlotState, search_bar::SearchBarState, tab_content::Modal,
     },
     writer::{
         Destination, JsonFormat, WriteToArrow, WriteToCsv, WriteToFile, WriteToJson, WriteToParquet,
@@ -31,12 +34,12 @@ pub enum AppAction {
     NoAction,
     ToggleBorders,
     DismissError,
-    DismissErrorAndShowPallete,
+    DismissErrorAndShowPalette,
     GotoLine(usize),
     SwitchToSchema,
     SwitchToTabulars,
 
-    TableInferColumns,
+    TableInferColumns(TypeInferer),
     TableDismissModal,
     TableScrollRight,
     TableScrollLeft,
@@ -65,18 +68,18 @@ pub enum AppAction {
     SheetScrollUp,
     SheetScrollDown,
 
-    PalleteGotoNext,
-    PalleteGotoPrev,
-    PalleteGotoStart,
-    PalleteGotoEnd,
-    PalleteDeleteNext,
-    PalleteDeletePrev,
-    PalleteInsert(char),
-    PalleteInsertSelectedOrCommit,
-    PalleteShow(String),
-    PalleteDeselectOrDismiss,
-    PalleteSelectPrevious,
-    PalleteSelectNext,
+    PaletteGotoNext,
+    PaletteGotoPrev,
+    PaletteGotoStart,
+    PaletteGotoEnd,
+    PaletteDeleteNext,
+    PaletteDeletePrev,
+    PaletteInsert(char),
+    PaletteInsertSelectedOrCommit,
+    PaletteShow(String),
+    PaletteDeselectOrDismiss,
+    PaletteSelectPrevious,
+    PaletteSelectNext,
 
     SearchFuzzyShow,
     SearchExactShow,
@@ -140,6 +143,15 @@ pub enum AppAction {
     SchemaOpenTable,
     SchemaUnloadTable,
 
+    DataFrameInfoScrollUp,
+    DataFrameInfoScrollDown,
+    DataFrameInfoShow,
+
+    ScatterPlot(String, String, Vec<String>),
+
+    HistogramPlot(String),
+
+    RegisterDataFrame(String),
     Help,
     Quit,
 }
@@ -161,19 +173,11 @@ pub fn execute(action: AppAction, app: &mut App) -> AppResult<Option<AppAction>>
         }
         AppAction::SwitchToTabulars => {
             app.switch_tabular();
-            if let Some(df) = app
-                .tabs()
-                .selected()
-                .map(TabularState::table)
-                .map(DataFrameTableState::data_frame)
-            {
-                sql().set_default(df.clone());
-            }
             Ok(None)
         }
-        AppAction::DismissErrorAndShowPallete => {
+        AppAction::DismissErrorAndShowPalette => {
             app.dismiss_error();
-            app.show_pallete("");
+            app.show_palette("");
             Ok(None)
         }
         AppAction::TableDismissModal => {
@@ -214,10 +218,10 @@ pub fn execute(action: AppAction, app: &mut App) -> AppResult<Option<AppAction>>
             }
             Ok(None)
         }
-        AppAction::TableInferColumns => {
+        AppAction::TableInferColumns(type_inferer) => {
             if let Some(tab) = app.tabs_mut().selected_mut() {
-                tab.table_mut().data_frame_mut().safe_infer_schema();
-                tab.table_mut().data_frame_mut().infer_datetime_columns();
+                let df = tab.table_mut().data_frame_mut();
+                type_inferer.update(df);
             }
             Ok(None)
         }
@@ -281,30 +285,34 @@ pub fn execute(action: AppAction, app: &mut App) -> AppResult<Option<AppAction>>
             Ok(None)
         }
         AppAction::TableSelect(select) => Ok(Some(AppAction::TableQuery(format!(
-            "SELECT {} FROM _",
-            select
+            "SELECT {select} FROM _"
         )))),
         AppAction::TableOrder(order) => Ok(Some(AppAction::TableQuery(format!(
-            "SELECT * FROM _ ORDER BY {}",
-            order
+            "SELECT * FROM _ ORDER BY {order}"
+        )))),
+        AppAction::TableFilter(filter) => Ok(Some(AppAction::TableQuery(format!(
+            "SELECT * FROM _ where {filter}"
         )))),
-        AppAction::TableFilter(filter) => Ok(Some(AppAction::TableQuery(
-            format! {"SELECT * FROM _ where {}", filter},
-        ))),
         AppAction::TableQuery(query) => {
-            let df = sql().execute(&query)?;
+            let df = sql().execute(
+                &query,
+                app.tabs()
+                    .selected()
+                    .map(TabContentState::table)
+                    .map(DataFrameTableState::data_frame)
+                    .cloned(),
+            )?;
             Ok(Some(AppAction::TableSetDataFrame(df)))
         }
         AppAction::TableSetDataFrame(df) => {
             if let Some(tab) = app.tabs_mut().selected_mut() {
                 tab.table_mut().set_data_frame(df.clone());
-                sql().set_default(df);
             }
             Ok(None)
         }
         AppAction::TableReset => {
             let query = match app.tabs_mut().selected().map(|ts| ts.table_type()) {
-                Some(TableType::Name(name)) => Some(format!("SELECT * FROM '{}'", name)),
+                Some(TableType::Name(name)) => Some(format!("SELECT * FROM '{name}'")),
                 Some(TableType::Query(query)) => Some(query.to_owned()),
                 Some(_) => None,
                 None => None,
@@ -315,7 +323,7 @@ pub fn execute(action: AppAction, app: &mut App) -> AppResult<Option<AppAction>>
             if let Some(sheet) = app
                 .tabs_mut()
                 .selected_mut()
-                .map(TabularState::modal_mut)
+                .map(TabContentState::modal_mut)
                 .and_then(Modal::sheet_mut)
             {
                 sheet.scroll_up();
@@ -326,7 +334,7 @@ pub fn execute(action: AppAction, app: &mut App) -> AppResult<Option<AppAction>>
             if let Some(sheet) = app
                 .tabs_mut()
                 .selected_mut()
-                .map(TabularState::modal_mut)
+                .map(TabContentState::modal_mut)
                 .and_then(Modal::sheet_mut)
             {
                 sheet.scroll_down();
@@ -335,13 +343,20 @@ pub fn execute(action: AppAction, app: &mut App) -> AppResult<Option<AppAction>>
         }
         AppAction::TabNewQuery(query) => {
             if sql().schema().iter().any(|(name, _)| name == &query) {
-                let df = sql().execute(&format!("SELECT * FROM '{}'", query))?;
+                let df = sql().execute(&format!("SELECT * FROM '{query}'"), None)?;
                 app.tabs_mut()
-                    .add(TabularState::new(df, TableType::Name(query)));
+                    .add(TabContentState::new(df, TableType::Name(query)));
             } else {
-                let df = sql().execute(&query)?;
+                let df = sql().execute(
+                    &query,
+                    app.tabs()
+                        .selected()
+                        .map(TabContentState::table)
+                        .map(DataFrameTableState::data_frame)
+                        .cloned(),
+                )?;
                 app.tabs_mut()
-                    .add(TabularState::new(df, TableType::Query(query)));
+                    .add(TabContentState::new(df, TableType::Query(query)));
             }
 
             Ok(Some(AppAction::TabSelect(
@@ -351,9 +366,6 @@ pub fn execute(action: AppAction, app: &mut App) -> AppResult<Option<AppAction>>
         AppAction::TabSelect(idx) => {
             let idx = idx.min(app.tabs_mut().len().saturating_sub(1));
             app.tabs_mut().select(idx);
-            if let Some(tabular) = app.tabs_mut().selected_mut() {
-                sql().set_default(tabular.table_mut().data_frame().clone());
-            }
             app.switch_tabular();
             Ok(None)
         }
@@ -437,7 +449,7 @@ pub fn execute(action: AppAction, app: &mut App) -> AppResult<Option<AppAction>>
             for (name, df) in frames {
                 let name = sql().register(&name, df.clone(), source.clone());
                 app.tabs_mut()
-                    .add(TabularState::new(df, TableType::Name(name)));
+                    .add(TabContentState::new(df, TableType::Name(name)));
             }
             Ok(None)
         }
@@ -446,7 +458,7 @@ pub fn execute(action: AppAction, app: &mut App) -> AppResult<Option<AppAction>>
             for (name, df) in frames {
                 let name = sql().register(&name, df.clone(), source.clone());
                 app.tabs_mut()
-                    .add(TabularState::new(df, TableType::Name(name)));
+                    .add(TabContentState::new(df, TableType::Name(name)));
             }
             Ok(Some(AppAction::TabSelect(
                 app.tabs_mut().len().saturating_sub(1),
@@ -462,7 +474,7 @@ pub fn execute(action: AppAction, app: &mut App) -> AppResult<Option<AppAction>>
             for (name, df) in frames {
                 let name = sql().register(&name, df.clone(), source.clone());
                 app.tabs_mut()
-                    .add(TabularState::new(df, TableType::Name(name)));
+                    .add(TabContentState::new(df, TableType::Name(name)));
             }
             Ok(Some(AppAction::TabSelect(
                 app.tabs_mut().len().saturating_sub(1),
@@ -473,7 +485,7 @@ pub fn execute(action: AppAction, app: &mut App) -> AppResult<Option<AppAction>>
             for (name, df) in frames {
                 let name = sql().register(&name, df.clone(), source.clone());
                 app.tabs_mut()
-                    .add(TabularState::new(df, TableType::Name(name)));
+                    .add(TabContentState::new(df, TableType::Name(name)));
             }
             Ok(Some(AppAction::TabSelect(
                 app.tabs_mut().len().saturating_sub(1),
@@ -484,7 +496,7 @@ pub fn execute(action: AppAction, app: &mut App) -> AppResult<Option<AppAction>>
             for (name, df) in frames {
                 let name = sql().register(&name, df.clone(), source.clone());
                 app.tabs_mut()
-                    .add(TabularState::new(df, TableType::Name(name)));
+                    .add(TabContentState::new(df, TableType::Name(name)));
             }
             Ok(Some(AppAction::TabSelect(
                 app.tabs_mut().len().saturating_sub(1),
@@ -506,7 +518,7 @@ pub fn execute(action: AppAction, app: &mut App) -> AppResult<Option<AppAction>>
             for (name, df) in frames {
                 let name = sql().register(&name, df.clone(), source.clone());
                 app.tabs_mut()
-                    .add(TabularState::new(df, TableType::Name(name)));
+                    .add(TabContentState::new(df, TableType::Name(name)));
             }
             Ok(Some(AppAction::TabSelect(
                 app.tabs_mut().len().saturating_sub(1),
@@ -516,7 +528,7 @@ pub fn execute(action: AppAction, app: &mut App) -> AppResult<Option<AppAction>>
             if let Some(sb) = app
                 .tabs_mut()
                 .selected_mut()
-                .map(TabularState::modal_mut)
+                .map(TabContentState::modal_mut)
                 .and_then(Modal::search_bar_mut)
             {
                 sb.goto_next();
@@ -527,7 +539,7 @@ pub fn execute(action: AppAction, app: &mut App) -> AppResult<Option<AppAction>>
             if let Some(sb) = app
                 .tabs_mut()
                 .selected_mut()
-                .map(TabularState::modal_mut)
+                .map(TabContentState::modal_mut)
                 .and_then(Modal::search_bar_mut)
             {
                 sb.goto_prev();
@@ -538,7 +550,7 @@ pub fn execute(action: AppAction, app: &mut App) -> AppResult<Option<AppAction>>
             if let Some(sb) = app
                 .tabs_mut()
                 .selected_mut()
-                .map(TabularState::modal_mut)
+                .map(TabContentState::modal_mut)
                 .and_then(Modal::search_bar_mut)
             {
                 sb.goto_start();
@@ -549,7 +561,7 @@ pub fn execute(action: AppAction, app: &mut App) -> AppResult<Option<AppAction>>
             if let Some(sb) = app
                 .tabs_mut()
                 .selected_mut()
-                .map(TabularState::modal_mut)
+                .map(TabContentState::modal_mut)
                 .and_then(Modal::search_bar_mut)
             {
                 sb.goto_end();
@@ -560,7 +572,7 @@ pub fn execute(action: AppAction, app: &mut App) -> AppResult<Option<AppAction>>
             if let Some(sb) = app
                 .tabs_mut()
                 .selected_mut()
-                .map(TabularState::modal_mut)
+                .map(TabContentState::modal_mut)
                 .and_then(Modal::search_bar_mut)
             {
                 sb.delete_next();
@@ -571,7 +583,7 @@ pub fn execute(action: AppAction, app: &mut App) -> AppResult<Option<AppAction>>
             if let Some(sb) = app
                 .tabs_mut()
                 .selected_mut()
-                .map(TabularState::modal_mut)
+                .map(TabContentState::modal_mut)
                 .and_then(Modal::search_bar_mut)
             {
                 sb.delete_prev();
@@ -582,7 +594,7 @@ pub fn execute(action: AppAction, app: &mut App) -> AppResult<Option<AppAction>>
             if let Some(sb) = app
                 .tabs_mut()
                 .selected_mut()
-                .map(TabularState::modal_mut)
+                .map(TabContentState::modal_mut)
                 .and_then(Modal::search_bar_mut)
             {
                 sb.insert(c);
@@ -625,7 +637,7 @@ pub fn execute(action: AppAction, app: &mut App) -> AppResult<Option<AppAction>>
             if let Some(idx) = idx {
                 Ok(Some(AppAction::TabSelect(idx)))
             } else {
-                app.tabs_mut().add(TabularState::new(
+                app.tabs_mut().add(TabContentState::new(
                     commands_help_data_frame(),
                     TableType::Help,
                 ));
@@ -680,64 +692,64 @@ pub fn execute(action: AppAction, app: &mut App) -> AppResult<Option<AppAction>>
             }
             Ok(None)
         }
-        AppAction::PalleteGotoNext => {
-            if let Some(pallete) = app.pallete_mut() {
-                pallete.input().goto_next();
+        AppAction::PaletteGotoNext => {
+            if let Some(palette) = app.palette_mut() {
+                palette.input().goto_next();
             }
             Ok(None)
         }
-        AppAction::PalleteGotoPrev => {
-            if let Some(pallete) = app.pallete_mut() {
-                pallete.input().goto_prev();
+        AppAction::PaletteGotoPrev => {
+            if let Some(palette) = app.palette_mut() {
+                palette.input().goto_prev();
             }
             Ok(None)
         }
-        AppAction::PalleteGotoStart => {
-            if let Some(pallete) = app.pallete_mut() {
-                pallete.input().goto_start();
+        AppAction::PaletteGotoStart => {
+            if let Some(palette) = app.palette_mut() {
+                palette.input().goto_start();
             }
             Ok(None)
         }
-        AppAction::PalleteGotoEnd => {
-            if let Some(pallete) = app.pallete_mut() {
-                pallete.input().goto_end();
+        AppAction::PaletteGotoEnd => {
+            if let Some(palette) = app.palette_mut() {
+                palette.input().goto_end();
             }
             Ok(None)
         }
-        AppAction::PalleteDeleteNext => {
-            if let Some(pallete) = app.pallete_mut() {
-                pallete.input().delete_next();
+        AppAction::PaletteDeleteNext => {
+            if let Some(palette) = app.palette_mut() {
+                palette.input().delete_next();
             }
             Ok(None)
         }
-        AppAction::PalleteDeletePrev => {
-            if let Some(pallete) = app.pallete_mut() {
-                pallete.input().delete_prev();
+        AppAction::PaletteDeletePrev => {
+            if let Some(palette) = app.palette_mut() {
+                palette.input().delete_prev();
             }
             Ok(None)
         }
-        AppAction::PalleteInsert(c) => {
-            if let Some(pallete) = app.pallete_mut() {
-                pallete.input().insert(c);
-                pallete.list().select(None);
+        AppAction::PaletteInsert(c) => {
+            if let Some(palette) = app.palette_mut() {
+                palette.input().insert(c);
+                palette.list().select(None);
             }
             Ok(None)
         }
-        AppAction::PalleteInsertSelectedOrCommit => {
+        AppAction::PaletteInsertSelectedOrCommit => {
             if let Some(selected) = app
-                .pallete_mut()
-                .and_then(|pallete| pallete.list().selected())
+                .palette_mut()
+                .and_then(|palette| palette.list().selected())
             {
                 if let Some(cmd) = app.history_mut().get(selected).map(String::to_owned) {
-                    if let Some(pallete) = app.pallete_mut() {
-                        pallete.set_input(cmd);
-                        pallete.list().select(None);
+                    if let Some(palette) = app.palette_mut() {
+                        palette.set_input(cmd);
+                        palette.list().select(None);
                     }
                 }
                 Ok(None)
-            } else if let Some(cmd) = app.hide_pallete() {
+            } else if let Some(cmd) = app.hide_palette() {
                 if cmd.is_empty() {
-                    Ok(Some(AppAction::PalleteDeselectOrDismiss))
+                    Ok(Some(AppAction::PaletteDeselectOrDismiss))
                 } else {
                     app.history_mut().push(cmd.clone());
                     parse_into_action(cmd).map(Some)
@@ -746,29 +758,29 @@ pub fn execute(action: AppAction, app: &mut App) -> AppResult<Option<AppAction>>
                 Ok(None)
             }
         }
-        AppAction::PalleteShow(text) => {
-            app.show_pallete(text);
+        AppAction::PaletteShow(text) => {
+            app.show_palette(text);
             Ok(None)
         }
-        AppAction::PalleteDeselectOrDismiss => {
-            if let Some(pallete) = app.pallete_mut() {
-                if pallete.list().selected().is_some() {
-                    pallete.list().select(None);
+        AppAction::PaletteDeselectOrDismiss => {
+            if let Some(palette) = app.palette_mut() {
+                if palette.list().selected().is_some() {
+                    palette.list().select(None);
                 } else {
-                    app.hide_pallete();
+                    app.hide_palette();
                 }
             }
             Ok(None)
         }
-        AppAction::PalleteSelectPrevious => {
-            if let Some(pallete) = app.pallete_mut() {
-                pallete.list().select_previous();
+        AppAction::PaletteSelectPrevious => {
+            if let Some(palette) = app.palette_mut() {
+                palette.list().select_previous();
             }
             Ok(None)
         }
-        AppAction::PalleteSelectNext => {
-            if let Some(pallete) = app.pallete_mut() {
-                pallete.list().select_next();
+        AppAction::PaletteSelectNext => {
+            if let Some(palette) = app.palette_mut() {
+                palette.list().select_next();
             }
             Ok(None)
         }
@@ -798,9 +810,14 @@ pub fn execute(action: AppAction, app: &mut App) -> AppResult<Option<AppAction>>
         }
         AppAction::SchemaFieldsScrollUp => {
             if app.content() == &Content::Schema {
-                *app.schema_mut().fields_mut().table_state_mut().offset_mut() = app
+                *app.schema_mut()
+                    .data_frame_info_mut()
+                    .field_info_mut()
+                    .table_state_mut()
+                    .offset_mut() = app
                     .schema()
-                    .fields()
+                    .data_frame_info()
+                    .field_info()
                     .table_state()
                     .offset()
                     .saturating_sub(1);
@@ -809,9 +826,14 @@ pub fn execute(action: AppAction, app: &mut App) -> AppResult<Option<AppAction>>
         }
         AppAction::SchemaFieldsScrollDown => {
             if app.content() == &Content::Schema {
-                *app.schema_mut().fields_mut().table_state_mut().offset_mut() = app
+                *app.schema_mut()
+                    .data_frame_info_mut()
+                    .field_info_mut()
+                    .table_state_mut()
+                    .offset_mut() = app
                     .schema()
-                    .fields()
+                    .data_frame_info()
+                    .field_info()
                     .table_state()
                     .offset()
                     .saturating_add(1);
@@ -895,5 +917,97 @@ pub fn execute(action: AppAction, app: &mut App) -> AppResult<Option<AppAction>>
                 Ok(None)
             }
         }
+        AppAction::RegisterDataFrame(name) => {
+            if sql().schema().iter().map(|(name, _)| name).contains(&name) {
+                Err(anyhow!("Data frame with name '{}' already exists.", &name))
+            } else {
+                if let Some(data_frame) = app
+                    .tabs()
+                    .selected()
+                    .map(TabContentState::table)
+                    .map(DataFrameTableState::data_frame)
+                    .cloned()
+                {
+                    sql().register(&name, data_frame, crate::misc::sql::Source::User);
+                }
+
+                Ok(None)
+            }
+        }
+        AppAction::DataFrameInfoScrollUp => {
+            if let Some(tabular) = app.tabs_mut().selected_mut() {
+                if let Modal::DataFrameInfo(data_frame_info_state) = tabular.modal_mut() {
+                    *data_frame_info_state
+                        .field_info_mut()
+                        .table_state_mut()
+                        .offset_mut() = data_frame_info_state
+                        .field_info()
+                        .table_state()
+                        .offset()
+                        .saturating_sub(1);
+                }
+            }
+            Ok(None)
+        }
+        AppAction::DataFrameInfoScrollDown => {
+            if let Some(tabular) = app.tabs_mut().selected_mut() {
+                if let Modal::DataFrameInfo(data_frame_info_state) = tabular.modal_mut() {
+                    *data_frame_info_state
+                        .field_info_mut()
+                        .table_state_mut()
+                        .offset_mut() = data_frame_info_state
+                        .field_info()
+                        .table_state()
+                        .offset()
+                        .saturating_add(1);
+                }
+            }
+            Ok(None)
+        }
+        AppAction::DataFrameInfoShow => {
+            if let Some(tabular) = app.tabs_mut().selected_mut() {
+                *tabular.modal_mut() = Modal::DataFrameInfo(DataFrameInfoState::default());
+            }
+            Ok(None)
+        }
+        AppAction::ScatterPlot(x_lab, y_lab, group_by) => {
+            if let Some(tab_content) = app.tabs_mut().selected_mut() {
+                let df = tab_content.table().data_frame();
+                if group_by.is_empty() {
+                    let data = df.scatter_plot_data(&x_lab, &y_lab)?;
+                    *tab_content.modal_mut() =
+                        Modal::ScatterPlot(ScatterPlotState::new(x_lab, y_lab, vec![data])?)
+                } else {
+                    let mut groups = Vec::new();
+                    let mut data = Vec::new();
+                    for df in df.partition_by(&group_by, true)? {
+                        let name = group_by
+                            .iter()
+                            .map(|col| {
+                                df.column(col)
+                                    .and_then(|column| column.get(0))
+                                    .map(IntoString::into_single_line)
+                                    .unwrap_or("null".to_owned())
+                            })
+                            .join(" - ");
+                        groups.push(name);
+                        data.push(df.scatter_plot_data(&x_lab, &y_lab)?);
+                    }
+                    *tab_content.modal_mut() = Modal::ScatterPlot(
+                        ScatterPlotState::new(x_lab, y_lab, data)?.groups(groups),
+                    )
+                }
+            }
+            Ok(None)
+        }
+        AppAction::HistogramPlot(group_by) => {
+            if let Some(tab_content) = app.tabs_mut().selected_mut() {
+                let df = tab_content.table().data_frame();
+                *tab_content.modal_mut() = Modal::HistogramPlot(HistogramPlotState::new(
+                    df.histogram_plot_data(&group_by, 24)?,
+                ))
+            }
+            Ok(None)
+        }
     }
 }
