@@ -1,8 +1,9 @@
 use anyhow::{Ok, anyhow};
+use itertools::Itertools;
 use polars::{df, frame::DataFrame};
 use std::{collections::HashMap, sync::OnceLock};
 
-use crate::{AppResult, handler::action::AppAction};
+use crate::{AppResult, handler::action::AppAction, misc::type_inferer::TypeInferer};
 
 pub fn parse_into_action(cmd: impl AsRef<str>) -> AppResult<AppAction> {
     let (s1, s2) = cmd.as_ref().split_once(' ').unwrap_or((cmd.as_ref(), ""));
@@ -39,10 +40,10 @@ pub fn commands_help_data_frame() -> DataFrame {
     .unwrap()
 }
 
-static REGISTERY: OnceLock<Registery> = OnceLock::new();
+static REGISTRY: OnceLock<Registry> = OnceLock::new();
 
-fn registary() -> &'static Registery {
-    REGISTERY.get_or_init(|| {
+fn registary() -> &'static Registry {
+    REGISTRY.get_or_init(|| {
         ENTRIES
             .iter()
             .flat_map(|cmd| {
@@ -59,7 +60,7 @@ fn registary() -> &'static Registery {
 }
 
 type ParseFn = fn(&str) -> AppResult<AppAction>;
-type Registery = HashMap<&'static str, ParseFn>;
+type Registry = HashMap<&'static str, ParseFn>;
 
 enum Prefix {
     Long(&'static str),
@@ -89,7 +90,7 @@ struct Entry {
     parser: ParseFn,
 }
 
-static ENTRIES: [Entry; 18] = [
+static ENTRIES: [Entry; 21] = [
     Entry {
         prefix: Prefix::ShortAndLong("Q", "query"),
         usage: "Q <query>",
@@ -99,7 +100,7 @@ static ENTRIES: [Entry; 18] = [
     Entry {
         prefix: Prefix::ShortAndLong("q", "quit"),
         usage: "q",
-        description: "Close all tabls and quit Tabiew",
+        description: "Close all tables and quit Tabiew",
         parser: |_| Ok(AppAction::Quit),
     },
     Entry {
@@ -169,7 +170,7 @@ static ENTRIES: [Entry; 18] = [
     Entry {
         prefix: Prefix::Long("schema"),
         usage: "schema",
-        description: "Show loaded data frame(s) and their schmea(s)",
+        description: "Show loaded data frame(s) and their schema(s)",
         parser: |_| Ok(AppAction::SwitchToSchema),
     },
     Entry {
@@ -197,10 +198,55 @@ static ENTRIES: [Entry; 18] = [
         parser: |query| Ok(AppAction::TabSelect(query.parse()?)),
     },
     Entry {
-        prefix: Prefix::Long("infer-types"),
-        usage: "infer-types",
+        prefix: Prefix::Long("infer"),
+        usage: "infer <types>",
         description: "Perform extra processing to infer column types",
-        parser: |_| Ok(AppAction::TableInferColumns),
+        parser: |args| {
+            let ti = args
+                .split(' ')
+                .try_fold(TypeInferer::default(), |ti, slice| match slice {
+                    "int" => Ok(ti.int()),
+                    "float" => Ok(ti.float()),
+                    "boolean" => Ok(ti.boolean()),
+                    "date" => Ok(ti.date()),
+                    "datetime" => Ok(ti.datetime()),
+                    "all" => Ok(ti.int().float().boolean().date().datetime()),
+                    _ => Err(anyhow!("Invalid type {slice}")),
+                })?;
+            Ok(AppAction::TableInferColumns(ti))
+        },
+    },
+    Entry {
+        prefix: Prefix::Long("register"),
+        usage: "register <data_frame_name>",
+        description: "register current data frame to the SQL backend with the given name",
+        parser: |name| Ok(AppAction::RegisterDataFrame(name.to_owned())),
+    },
+    Entry {
+        prefix: Prefix::Long("scatter"),
+        usage: "scatter <x-axis> <y-axis> <group-by>",
+        description: "Draw a scatter plot given the axes",
+        parser: |args| {
+            let mut args = shell_words::split(args)?.into_iter();
+            let x = args.next().ok_or(anyhow!("require two axes"))?;
+            let y = args.next().ok_or(anyhow!("require two axes"))?;
+            let gb = args.collect_vec();
+            Ok(AppAction::ScatterPlot(x, y, gb))
+        },
+    },
+    Entry {
+        prefix: Prefix::Long("hist"),
+        usage: "hist <axes>",
+        description: "Draw a histogram plot given the axes",
+        parser: |col| {
+            //
+            let args = shell_words::split(col)?;
+            if args.len() == 1 {
+                Ok(AppAction::HistogramPlot(args.into_iter().next().unwrap()))
+            } else {
+                Err(anyhow!("histogram should be supplied with only one column"))
+            }
+        },
     },
     export::entry(),
     import::entry(),
