@@ -12,13 +12,13 @@ use tabiew::handler::command::parse_into_action;
 use tabiew::handler::event::{Event, EventHandler};
 use tabiew::handler::key::KeyHandler;
 use tabiew::misc::globals::{set_theme, sql};
-use tabiew::misc::polars_ext::InferDatetimeColumns;
+use tabiew::misc::type_inferer::TypeInferer;
 use tabiew::reader::{BuildReader, Source};
 
 use tabiew::tui::theme::{
     Argonaut, Catppuccin, Chakra, Monokai, Nord, Terminal, Theme, TokyoNight,
 };
-use tabiew::tui::{TableType, TabularState};
+use tabiew::tui::{TabContentState, TableType};
 use tabiew::{AppResult, tui};
 
 use tabiew::misc::history::{History, enforce_line_limit};
@@ -42,13 +42,13 @@ fn main() -> AppResult<()> {
         return Ok(());
     }
 
+    let type_infer = TypeInferer::from_args(&args);
+
     // Load files to data frames
     let tabs = if args.files.is_empty() {
         let mut vec = Vec::new();
         for (name, mut df) in args.build_reader("")?.named_frames(Source::Stdin)? {
-            if args.infer_datetimes {
-                df.infer_datetime_columns();
-            }
+            type_infer.update(&mut df);
             vec.push((df.clone(), sql().register(&name, df, Source::Stdin)))
         }
         vec
@@ -57,13 +57,12 @@ fn main() -> AppResult<()> {
         for path in args.files.iter() {
             let source = Source::File(path.clone());
             let reader = args.build_reader(path)?;
-            let frames = reader
-                .named_frames(source.clone())
-                .unwrap_or_else(|err| panic!("{}", err));
+            let frames = reader.named_frames(source.clone()).unwrap_or_else(|err| {
+                eprintln!("tw: {err}");
+                std::process::exit(1)
+            });
             for (name, mut df) in frames {
-                if args.infer_datetimes {
-                    df.infer_datetime_columns();
-                }
+                type_infer.update(&mut df);
                 let name = sql().register(&name, df.clone(), source.clone());
                 vec.push((df, name))
             }
@@ -108,19 +107,11 @@ fn main() -> AppResult<()> {
 fn start_tui(tabs: Vec<(DataFrame, String)>, script: String, history: History) -> AppResult<()> {
     let tabs = tabs
         .into_iter()
-        .map(|(df, name)| TabularState::new(df, TableType::Name(name)))
+        .map(|(df, name)| TabContentState::new(df, TableType::Name(name)))
         .collect();
     let keybind = KeyHandler::default();
     let mut app = App::new(tabs, history);
 
-    // Set default data frame to the first tab
-    sql().set_default(
-        app.tabs_mut()
-            .selected()
-            .map(|tab| tab.table().data_frame().clone())
-            .unwrap(),
-    );
-
     // Initialize the terminal user interface.
     let mut tui = tui::Terminal::new(
         ratatui::Terminal::new(CrosstermBackend::new(io::stderr()))?,
@@ -133,9 +124,14 @@ fn start_tui(tabs: Vec<(DataFrame, String)>, script: String, history: History) -
 
     // Run startup script
     for line in script.lines().filter(|line| !line.is_empty()) {
-        let action = parse_into_action(line)
-            .unwrap_or_else(|err| panic!("Error in startup script: {}", err));
-        execute(action, &mut app).unwrap_or_else(|err| panic!("Error in startup script: {}", err));
+        let action = parse_into_action(line).unwrap_or_else(|err| {
+            eprintln!("tw: Error in startup script: {err}");
+            std::process::exit(1);
+        });
+        execute(action, &mut app).unwrap_or_else(|err| {
+            eprintln!("tw: Error in startup script: {err}");
+            std::process::exit(1);
+        });
     }
 
     // Main loop
