@@ -1,22 +1,25 @@
-pub mod command_pallete;
+pub mod command_palette;
+pub mod data_frame_field_info;
+pub mod data_frame_info;
+pub mod data_frame_meta_info;
+pub mod data_frame_names;
 pub mod data_frame_table;
 pub mod error_popup;
-pub mod field_info_table;
+pub mod histogram_plot;
 pub mod input;
+pub mod scatter_plot;
 pub mod schema;
 pub mod search_bar;
 pub mod sheet;
 pub mod side_panel;
 pub mod status_bar;
 pub mod tab;
-pub mod table_info_table;
-pub mod table_names_table;
-pub mod tabular;
+pub mod tab_content;
 pub mod terminal;
 pub mod theme;
 mod utils;
 
-pub use tabular::{TableType, TabularState};
+pub use tab_content::{TabContentState, TableType};
 pub use terminal::Terminal;
 pub use theme::Styler;
 
