@@ -12,6 +12,7 @@ pub trait Styler {
     fn text(&self) -> Style;
     fn subtext(&self) -> Style;
     fn error(&self) -> Style;
+    fn graph(&self, idx: usize) -> Style;
 }
 
 pub trait SixColorsTwoRowsStyler {
@@ -24,8 +25,8 @@ pub trait SixColorsTwoRowsStyler {
     const DARK_COLORS: [Color; 6];
 
     const ROW_BACKGROUNDS: [Color; 2];
-    const HIGHTLIGHT_BACKGROUND: Color;
-    const HIGHTLIGHT_FOREGROUND: Color;
+    const HIGHLIGHT_BACKGROUND: Color;
+    const HIGHLIGHT_FOREGROUND: Color;
 
     const STATUS_BAR_ERROR: Color;
 }
@@ -52,8 +53,8 @@ where
 
     fn highlight(&self) -> Style {
         Style::new()
-            .bg(Self::HIGHTLIGHT_BACKGROUND)
-            .fg(Self::HIGHTLIGHT_FOREGROUND)
+            .bg(Self::HIGHLIGHT_BACKGROUND)
+            .fg(Self::HIGHLIGHT_FOREGROUND)
     }
 
     fn tag(&self, idx: usize) -> Style {
@@ -65,12 +66,12 @@ where
     fn block(&self) -> Style {
         Style::default()
             .bg(Self::BACKGROUND)
-            .fg(Self::HIGHTLIGHT_BACKGROUND)
+            .fg(Self::HIGHLIGHT_BACKGROUND)
     }
 
     fn block_tag(&self) -> Style {
         Style::default()
-            .bg(Self::HIGHTLIGHT_BACKGROUND)
+            .bg(Self::HIGHLIGHT_BACKGROUND)
             .fg(Self::LIGHT_BACKGROUND)
     }
 
@@ -89,6 +90,12 @@ where
             .bg(Self::STATUS_BAR_ERROR)
             .fg(Self::FOREGROUND)
     }
+
+    fn graph(&self, idx: usize) -> Style {
+        Style::default()
+            .fg(Self::DARK_COLORS[idx % Self::DARK_COLORS.len()])
+            .bold()
+    }
 }
 
 #[derive(Debug, Default)]
@@ -136,8 +143,8 @@ impl SixColorsTwoRowsStyler for Monokai {
     ];
 
     const ROW_BACKGROUNDS: [Color; 2] = [Color::from_u32(0x00232024), Color::from_u32(0x001c191d)];
-    const HIGHTLIGHT_BACKGROUND: Color = Color::from_u32(0x00c89f2d);
-    const HIGHTLIGHT_FOREGROUND: Color = Self::BACKGROUND;
+    const HIGHLIGHT_BACKGROUND: Color = Color::from_u32(0x00c89f2d);
+    const HIGHLIGHT_FOREGROUND: Color = Self::BACKGROUND;
 
     const STATUS_BAR_ERROR: Color = Color::from_u32(0x00d02d00);
 }
@@ -166,8 +173,8 @@ impl SixColorsTwoRowsStyler for Argonaut {
     ];
 
     const ROW_BACKGROUNDS: [Color; 2] = [Color::from_u32(0x0011131b), Color::from_u32(0x0001030b)];
-    const HIGHTLIGHT_BACKGROUND: Color = Color::from_u32(0x00204a5b);
-    const HIGHTLIGHT_FOREGROUND: Color = Self::FOREGROUND;
+    const HIGHLIGHT_BACKGROUND: Color = Color::from_u32(0x00204a5b);
+    const HIGHLIGHT_FOREGROUND: Color = Self::FOREGROUND;
 
     const STATUS_BAR_ERROR: Color = Color::from_u32(0x00dd0000);
 }
@@ -197,8 +204,8 @@ impl SixColorsTwoRowsStyler for Nord {
     ];
 
     const ROW_BACKGROUNDS: [Color; 2] = [Color::from_u32(0x003B4252), Color::from_u32(0x00434C5E)];
-    const HIGHTLIGHT_BACKGROUND: Color = Color::from_u32(0x00DBBB7B);
-    const HIGHTLIGHT_FOREGROUND: Color = Color::from_u32(0x002E3440);
+    const HIGHLIGHT_BACKGROUND: Color = Color::from_u32(0x00DBBB7B);
+    const HIGHLIGHT_FOREGROUND: Color = Color::from_u32(0x002E3440);
     const STATUS_BAR_ERROR: Color = Color::from_u32(0x00BF616A);
 }
 
@@ -225,8 +232,8 @@ impl SixColorsTwoRowsStyler for Catppuccin {
         Color::from_u32(0x0089b4fa),
     ];
     const ROW_BACKGROUNDS: [Color; 2] = [Color::from_u32(0x00181825), Color::from_u32(0x001e1e2e)];
-    const HIGHTLIGHT_BACKGROUND: Color = Color::from_u32(0x00f9e2af);
-    const HIGHTLIGHT_FOREGROUND: Color = Color::from_u32(0x0011111b);
+    const HIGHLIGHT_BACKGROUND: Color = Color::from_u32(0x00f9e2af);
+    const HIGHLIGHT_FOREGROUND: Color = Color::from_u32(0x0011111b);
     const STATUS_BAR_ERROR: Color = Color::from_u32(0x00d36b98);
 }
 
@@ -253,8 +260,8 @@ impl SixColorsTwoRowsStyler for TokyoNight {
         Color::from_u32(0x0041a6b5),
     ];
     const ROW_BACKGROUNDS: [Color; 2] = [Color::from_u32(0x00292e42), Color::from_u32(0x0024283b)];
-    const HIGHTLIGHT_BACKGROUND: Color = Color::from_u32(0x00ffc777);
-    const HIGHTLIGHT_FOREGROUND: Color = Color::from_u32(0x001f2335);
+    const HIGHLIGHT_BACKGROUND: Color = Color::from_u32(0x00ffc777);
+    const HIGHLIGHT_FOREGROUND: Color = Color::from_u32(0x001f2335);
     const STATUS_BAR_ERROR: Color = Color::from_u32(0x00c53b53);
 }
 
@@ -280,8 +287,8 @@ impl SixColorsTwoRowsStyler for Chakra {
         Color::from_u32(0x0022c55e),
     ];
     const ROW_BACKGROUNDS: [Color; 2] = [Color::from_u32(0x0018181b), Color::from_u32(0x00111111)];
-    const HIGHTLIGHT_BACKGROUND: Color = Color::from_u32(0x00ca8a04);
-    const HIGHTLIGHT_FOREGROUND: Color = Color::from_u32(0x00fafafa);
+    const HIGHLIGHT_BACKGROUND: Color = Color::from_u32(0x00ca8a04);
+    const HIGHLIGHT_FOREGROUND: Color = Color::from_u32(0x00fafafa);
     const STATUS_BAR_ERROR: Color = Color::from_u32(0x00991919);
 }
 
@@ -333,6 +340,10 @@ impl Styler for Terminal {
     fn error(&self) -> Style {
         Style::default().bg(Color::Red).fg(Color::White)
     }
+
+    fn graph(&self, _idx: usize) -> Style {
+        Style::default().fg(Color::White)
+    }
 }
 
 #[derive(Debug, Default, Serialize, Deserialize)]
@@ -348,6 +359,7 @@ pub struct Theme {
     text: Style,
     subtext: Style,
     error: Style,
+    chart: Vec<Style>,
 }
 
 impl Theme {
@@ -401,6 +413,17 @@ impl Theme {
             error: Style::default()
                 .bg(Color::from_u32(0x00ff0000))
                 .fg(Color::from_u32(0x00ffffff)),
+            chart: vec![
+                Style::default()
+                    .bg(Color::from_u32(0x00000000))
+                    .fg(Color::from_u32(0x00ffff00)),
+                Style::default()
+                    .bg(Color::from_u32(0x00000000))
+                    .fg(Color::from_u32(0x00ff00ff)),
+                Style::default()
+                    .bg(Color::from_u32(0x00000000))
+                    .fg(Color::from_u32(0x0000ffff)),
+            ],
         }
     }
 }
@@ -445,4 +468,8 @@ impl super::Styler for Theme {
     fn error(&self) -> Style {
         self.error
     }
+
+    fn graph(&self, idx: usize) -> Style {
+        self.chart[idx % self.table_tags.len()]
+    }
 }
